@@ -3,11 +3,14 @@ import aiosqlite
 import asyncio
 import discord
 import os
+import re
 from discord.ext import commands
 from typing import Any, Awaitable, Callable, Optional, Union
 from .help_command import HelpCommand
-from .cogs.meta import *
+from .cogs.meta import Meta
 from .utils import *
+from .database import *
+
 
 class DiscordBot(commands.Bot):
     """
@@ -18,12 +21,13 @@ class DiscordBot(commands.Bot):
             self,
             prefix: Optional[str] = ".",
             color: Optional[Union[discord.Color, tuple, str]] = discord.Color.dark_theme(),
-            database: Optional[str] = None,
+            database: Optional[str] = "bot.db",
             members_intent: Optional[bool] = False,
             presences_intent: Optional[bool] = False,
             status: Optional[str] = "online",
             activity: Optional[str] = None,
             loop: Optional[asyncio.BaseEventLoop] = None,
+            custom_emojis: Optional[dict] = None,
             *args, **kwargs
     ):
         """Defines a new Discord Bot
@@ -38,8 +42,8 @@ class DiscordBot(commands.Bot):
             The default embed color to use for sending Discord embeds (defaults to `discord.Color.dark_theme()`)
             This parameter could be a `discord.Color` object, a hexadecimal color code, or an RGB tuple. The color will always
             end up being converted to a `discord.Color` object
-        database : Optional str (DEPRECIATED)
-            The name of the database file (defaults to "bot.db")
+        database : Optional str
+            The name of the bot's sqlite database file (defaults to "bot.db")
         members_intent : Optional bool
             Whether to requests the `members` Discord API Intent (defaults to False).
             If this is set to true, you will need to ensure your bot has the
@@ -55,10 +59,12 @@ class DiscordBot(commands.Bot):
             The "playing" activity to set for the bot (defaults to "with Cheesy | (PREFIX)help)
         loop : Optional asyncio.BaseEventLoop
             The asyncio event loop to set for the bot (defaults to the value returned by `asyncio.get_event_loop()`)
+        custom_emojis : Optional dict
+            A dictionary who's string key corresponds to a particular Discord emoji string. Used for `Discord.Bot.get_emoji`
         """
 
         self.loop = loop or asyncio.get_event_loop()
-        self.color = self.get_discord_color(color)
+        self.color = get_discord_color(color)
 
         # set intents
         intents = discord.Intents.default()
@@ -69,6 +75,11 @@ class DiscordBot(commands.Bot):
         if activity is None:
             activity = f"with Cheesy | {prefix}help"
 
+        # set database
+        self.database = Database(database)
+
+        self.is_actually_ready = False
+
         super().__init__(
             command_prefix=commands.when_mentioned_or(prefix),
             intents=intents,
@@ -78,10 +89,16 @@ class DiscordBot(commands.Bot):
             *args, **kwargs
         )
 
+        self.custom_emojis = custom_emojis
+
         self.add_cog(Meta(self))
-    
+
     async def on_ready(self):
-        print(f"{self.user} is ready!")
+        if not self.is_actually_ready:
+            # get current server id's
+            #server_ids = await self.database.query_all(Table("config").sql)
+            self.is_actually_ready = True
+            print(f"{self.user} is ready!")
     
     def run(self, token: Union[os.PathLike, str]):
         """Starts the bot
@@ -109,6 +126,37 @@ class DiscordBot(commands.Bot):
         except (aiohttp.ClientConnectorError, discord.HTTPException, discord.LoginFailure) as e:
             print(f"Unable to start bot: \"{type(e)}\" - \"{e}\"")
 
+    def get_emoji(self, id: Union[str, int]) -> Optional[discord.Emoji]:
+        """Returns a Discord Emoji
+
+        This method slightly differs from discord.py's implementation.
+        For starters, the bot will first look inside it's user-defined emoji reference (see `DiscordBot.custom_emojis`).
+        From there, the bot will continue to search for the emoji in it's cache similar to discord.py's implementation
+
+        Parameters
+        ----------
+        id : Union(str, int)
+            The Discord ID OR a key for `DiscordBot.custom_emojis`
+        
+        Returns
+        -------
+        Optional discord.Emoji
+        """
+
+        if isinstance(id, str):
+            # lookup key in the bot's custom emoji key
+            if self.custom_emojis is not None:
+                emoji_str = self.custom_emojis.get(id)
+                
+                if emoji_str is not None:
+                    def extract_emoji_id(emoji: str) -> int:
+                        return int(re.match(r'<a?:[a-zA-Z0-9\_]+:([0-9]+)>$', emoji).group(1))
+
+                    return super().get_emoji(extract_emoji_id(emoji_str))
+
+        # defaults to parent method should the above fail
+        return super().get_emoji(id)
+
     async def paginate(
         self,
         ctx: commands.Context,
@@ -203,34 +251,7 @@ class DiscordBot(commands.Bot):
             return discord.Status.online
         else:
             raise ValueError(f"Invalid status string \"{status}\"")
-
-    def get_discord_color(self, color: Union[discord.Color, tuple, str]) -> discord.Color:
-        """Returns a discord.Color from an RGB tuple or hex string
-        
-        The hex string parsing is case insensitive
-
-        Parameters
-        ----------
-        color: Union(tuple, str)
-
-        Raises
-        ------
-        TypeError if the color was not a discord.Color, tuple, or string
-
-        Returns
-        -------
-        A discord.Color object
-        """
-
-        if type(color) is tuple:
-            # assuming it's RGB
-            return discord.Color.from_rgb(color[0], color[1], color[2])
-        elif type(color) is str:
-            # code snippet taken from https://stackoverflow.com/a/29643643
-            return self.get_discord_color(tuple(int(color.lstrip("#")[i:i + 2], 16) for i in (0, 2, 4)))
-        else:
-            raise TypeError("Invalid Color type. Must be discord.Color, RGB tuple, or hex string")
-    
+    @commands.command
     @staticmethod
     async def _retrieve_entity(snowflake: int, func: Callable[[int], Any], coro: Awaitable[int]):
         """Lazily retrieves a discord object from a given discord id
@@ -280,7 +301,7 @@ class DiscordBot(commands.Bot):
         Returns
         -------
         An discord.abc.GuildChannel for guild-specific channels,
-        or discord.abc.PrivateChannel for user specific channels (dm's),
+        or discord.abc.PrivateChannel for user specific channels (DMs),
         or None if the channel could not be retrieved
         """
 
