@@ -2,6 +2,37 @@ import datetime
 import discord
 from discord.ext import commands
 from typing import List, Optional, Union
+from .constants import *
+
+
+def get_discord_color(color: Union[discord.Color, tuple, str]) -> discord.Color:
+    """Returns a discord.Color from an RGB tuple or hex string
+    
+    The hex string parsing is case insensitive
+
+    Parameters
+    ----------
+    color: Union(discord.Color, tuple, str)
+
+    Raises
+    ------
+    TypeError if the color was not a discord.Color, tuple, or string
+
+    Returns
+    -------
+    A discord.Color object
+    """
+
+    if type(color) is tuple:
+        # assuming it's RGB
+        return discord.Color.from_rgb(color[0], color[1], color[2])
+    elif type(color) is str:
+        # code snippet taken from https://stackoverflow.com/a/29643643
+        return get_discord_color(tuple(int(color.lstrip("#")[i:i + 2], 16) for i in (0, 2, 4)))
+    elif type(color) is discord.Color:
+        return color
+    else:
+        raise TypeError("Invalid Color type. Must be discord.Color, RGB tuple, or hex string")
 
 
 def get_image_url(entity: Union[discord.abc.User, discord.Guild, discord.Asset, str]):
@@ -64,7 +95,7 @@ def truncate(string: str, max_length: int, end: Optional[str] = "...") -> str:
     return truncated
 
 
-def chunkify_string(string: str, max_length: int) -> List[str]:
+def slice_string(string: str, max_length: int) -> List[str]:
     """Returns a list of strings of a particular maximum length
     
     Original solution taken from https://stackoverflow.com/a/18854817
@@ -72,9 +103,9 @@ def chunkify_string(string: str, max_length: int) -> List[str]:
     Parameters
     ----------
     string : str
-        The string to chunkify
+        The string to slice
     max_length : int
-        The maximum length for each string chunk
+        The maximum length for each string slice
     
     Returns
     -------
@@ -87,7 +118,7 @@ def chunkify_string(string: str, max_length: int) -> List[str]:
 def get_base_embed(
     title: Optional[str] = discord.Embed.Empty,
     description: Optional[str] = discord.Embed.Empty,
-    color: Optional[discord.Color] = discord.Embed.Empty,
+    color: Optional[Union[discord.Color, tuple, str]] = discord.Embed.Empty,
     timestamp: Optional[datetime.datetime] = datetime.datetime.utcnow(),
     author: Optional[Union[discord.abc.User, discord.Guild]] = None,
     author_name: Optional[str] = None,
@@ -110,8 +141,8 @@ def get_base_embed(
         The embed title
     description : Optional str
         The embed description
-    color : Optional discord.Color
-        The color to use for the embed
+    color : Optional Union(discord.Color, tuple, str)
+        The color to use for the embed. Can be a `discord.Color` object, RGB tuple, or hex string
     timestamp : Optional datetime.datetime
         The utc timestamp to use for the embed (defaults to the current utc timestamp)
     author : Optional Union(discord.abc.User, discord.Guild)
@@ -169,25 +200,19 @@ def get_base_embed(
     # override footer attributes
     if footer is not None:
         footer_text = str(footer)
-        footer_icon = footer.avatar_url if isinstance(footer, discord.abc.User) else author.icon_url
+        footer_icon = footer.avatar_url if isinstance(footer, discord.abc.User) else footer.icon_url
 
     # truncate neccessary fields
-    MAX_TITLE_LENGTH = 256
-    MAX_DESCRIPTION_LENGTH = 2048
-    MAX_FOOTER_TEXT_LENGTH = 2048
-    MAX_AUTHOR_NAME_LENGTH = 256
-    MAX_TOTAL_LENGTH = 6000
-
-    title = truncate(title, MAX_TITLE_LENGTH) if title is not discord.Embed.Empty else discord.Embed.Empty
-    description = truncate(description, MAX_DESCRIPTION_LENGTH) if description is not discord.Embed.Empty else discord.Embed.Empty
-    footer_text = truncate(footer_text, MAX_FOOTER_TEXT_LENGTH) if footer_text is not Empty else Empty
-    author_name = truncate(author_name, MAX_AUTHOR_NAME_LENGTH) if author_name is not None else None
+    title = truncate(title, MAX_EMBED_TITLE_LENGTH) if title is not discord.Embed.Empty else discord.Embed.Empty
+    description = truncate(description, MAX_EMBED_DESCRIPTION_LENGTH) if description is not discord.Embed.Empty else discord.Embed.Empty
+    footer_text = truncate(footer_text, MAX_EMBED_FOOTER_TEXT_LENGTH) if footer_text is not Empty else Empty
+    author_name = truncate(author_name, MAX_EMBED_AUTHOR_NAME_LENGTH) if author_name is not None else None
 
     # construct embed
     embed = discord.Embed(
         title=title,
         description=description,
-        color=color,
+        color=get_discord_color(color),
         timestamp=timestamp,
         url=url,
     )
@@ -216,8 +241,8 @@ def get_base_embed(
         embed.set_image(get_image_url(image))
     
     # check max length
-    if len(embed) > MAX_TOTAL_LENGTH:
-        raise ValueError(f"Base embed character count of {len(embed)} is over limit of {MAX_TOTAL_LENGTH}")
+    if len(embed) > MAX_EMBED_TOTAL_LENGTH:
+        raise ValueError(f"Base embed character count of {len(embed)} is over limit of {MAX_EMBED_TOTAL_LENGTH}")
     
     return embed
 
